% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                       %
% %    Matlab function for lecture                                        %
% %    "Correlation analysis of multivariate time series and              %
% %     Principle Component Analysis"                                     %
% %    by Christian Rummel                                                %
% %                                                                       %
% %    BENESCO lecture series                                             %
% %    "Signal Analysis and Brain Oscillations in Health and Disease"     %
% %    Bern, 2016/04/01                                                   %
% %    Bern, 2017/03/31                                                   %
% %                                                                       %
% %    - windowed analysis of EEG data                                    %
% %    - covariance and correlation matrices are compared                 %
% %                                                                       %
% %    usage:                                                             %
% %    CovVsCorr (tme,EEG, T,dT, dspl)                                    %
% %                                                                       %
% %    input values:                                                      %
% %    tme    vector containing time from file start in seconds           %
% %    EEG    data matrix (L rows: temporal samples, M columns: chann.)   %
% %    T      size of analysis window in samples                          %
% %    dT     shift of analysis window in samples                         %
% %    dspl   flag for display of intermediate results (if >0)            %
% %                                                                       %
% %    literature:                                                        %
% %    - Schindler et al. (2007), Brain 131, 65-77                        %
% %    - Schindler et al. (2007), Clin. Neurophysiol. 118, 1955-1968      %
% %                                                                       %
% %    written by C. Rummel, 2016/03/23                                   %
% %    modified 2017/03/29                                                %
% %                                                                       %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function CovVsCorr (tme,EEG, T,dT, dspl)

    close all
    clc

    chan  = [1 13 25 33 41 47 53 59 64]; % channels selected for matrix display
%     chan  = 1:9;

    [L M] = size(EEG)           % L: number of rows, temporal samples
                                % M: number of columns, signal channels

    cov_mat_all  = cov(EEG);
    cov_scal_all = max(abs(cov_mat_all(:)));
                                
% loop for windowed analysis of EEG data

    t = 1;
    t_max = floor((L-T)/dT+1);
    i_max = ceil(sqrt(t_max));
    j_max = ceil(t_max/i_max);

    for t = 1:t_max

% select data window and calculate column-wise signal standard deviations
    
        tme_win = tme((t-1)*dT+1:(t-1)*dT+T);
        EEG_win = EEG((t-1)*dT+1:(t-1)*dT+T,:);
        EEG_std = std(EEG_win,0,1);
    
% optional display of signals

        if dspl>0
            figure(10);
            hold on
            for i = 1:M
                if ismember(i,chan)
                    plot(tme_win,EEG_win(:,i)/max(EEG_std)/3-i,'r');
                else
                    plot(tme_win,EEG_win(:,i)/max(EEG_std)/3-i,'b');
                end
            end
            title('scaled signals');
        end

% calculate covariance matrix

        cov_mat  = cov (EEG_win);
        cov_scal = max(abs(cov_mat(:)));

% optional display of covariance matrix
        
        if dspl>0
            figure(20)
            for i = 1:length(chan)
                for j = 1:length(chan)
                    ax = axes('position',[0.05+(i-1)*0.95/length(chan) 1-j*0.95/length(chan) 0.9/length(chan) 0.9/length(chan)]);
                    plot(ax,EEG_win(:,chan(i)),EEG_win(:,chan(j)),'r');
                    xlim([-4*max(EEG_std) 4*max(EEG_std)]);
                    ylim([-4*max(EEG_std) 4*max(EEG_std)]);
                    set(ax,'XTick',[],'YTick',[]);
                end
            end

            figure(30);
            imagesc(cov_mat,[-cov_scal cov_scal]); colorbar;
            title('covariance matrix');
        end

% optional display of normalized signals
    
        if dspl>0
            figure(40)
            hold on
            for i = 1:M
                if ismember(i,chan)
                    plot(tme_win,zscore(EEG_win(:,i))/5-i,'r');
                else
                    plot(tme_win,zscore(EEG_win(:,i))/5-i,'b');
                end
            end
            title('normalized signals');
        end

% calculate Pearson's correlation matrix

        corr_mat = corr(EEG_win);

% optional display of Pearson's correlation matrix

        if dspl>0
            figure(50);
            imagesc(corr_mat,[-1 1]); colorbar;
            title('Pearson correlation matrix');
        end

% figures for temporal evolution of covariance and correlation matrices
% time evolves row-wise from left to right

        [i,j] = ind2sub([i_max,j_max],t);

        figure(60)
        axes('position',[0.05+(i-1)*0.95/i_max 1-j*0.95/j_max 0.9/i_max 0.9/j_max]);
        imagesc(cov_mat,[-cov_scal_all cov_scal_all]); 
        set(gca,'XTick',[],'YTick',[]);

        figure(70)
        axes('position',[0.05+(i-1)*0.95/i_max 1-j*0.95/j_max 0.9/i_max 0.9/j_max]);
        imagesc(corr_mat,[-1 1]); 
        set(gca,'XTick',[],'YTick',[]);
        
% optionally wait for key press and continue with next window
    
        if dspl>0
            disp('press key to continue!');
            pause

            close 10
            close 20
            close 30
            close 40
            close 50
        end
    end
    
end

