% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                       %
% %    Matlab function for lecture                                        %
% %    "Correlation analysis of multivariate time series and              %
% %     Principle Component Analysis"                                     %
% %    by Christian Rummel                                                %
% %                                                                       %
% %    BENESCO lecture series                                             %
% %    "Signal Analysis and Brain Oscillations in Health and Disease"     %
% %    Bern, 2016/04/01                                                   %
% %    Bern, 2017/03/31                                                   %
% %                                                                       %
% %    - analysis of full EEG data (no windowing)                         %
% %    - principal component analysis is introduced                       %
% %                                                                       %
% %    usage:                                                             %
% %    PrinCompAna (tme,EEG, PCA_del)                                     %
% %                                                                       %
% %    input values:                                                      %
% %    tme        vector containing time from file start in seconds       %
% %    EEG        data matrix (L rows: temp. samples, M columns: chann.)  %
% %    PCA_del    vector of deleted PCA components                        %
% %                                                                       %
% %    written by C. Rummel, 2016/03/23                                   %
% %    modified 2017/03/29                                                %
% %                                                                       %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function PrinCompAna (tme,EEG, PCA_del)

    close all
    clc

    [L M] = size(EEG)           % L: number of rows, temporal samples
                                % M: number of columns, signal channels

% display covariance and correlation matrix

    cov_mat  = cov(EEG);
    cov_scal = max(abs(cov_mat(:)));

    figure(10);
    subplot(1,2,1);
    imagesc(cov_mat,[-cov_scal cov_scal]); colorbar;
    title('covariance matrix of signals');
    subplot(1,2,2);
    imagesc(corr(EEG),[-1 1]); colorbar;
    title('correlation matrix of signals');

% diagonalize covariance matrix    
    
    [V,D]    = eig(cov_mat);
    lambda   = D(1:M+1:M^2);

    figure(20);
    subplot(1,2,1);
    bar(lambda);
    title('covariance: eigenvalues');
    subplot(1,2,2);
    imagesc(V,[-1 1]); colorbar;
    title('covariance: eigenvectors');
    
% principal component analysis based on covariance matrix
% "pedestrian version"

    PCAcov = EEG*V;

    figure(30);
    subplot(1,2,1);
    hold on
    for i = 1:M
        plot(tme,zscore(EEG(:,i))/10-i);
    end
    title('normalized signals');
    subplot(1,2,2);
    hold on
    for i = 1:M
        plot(tme,zscore(PCAcov(:,i))/10-i);
    end
    title('normalized principal components');

    cov_scal = max(max(abs(cov(PCAcov)),[],1),[],2);
    
% principal component analysis based on correlation matrix
% Matlab function pca.m

    [coeff,score,latent] = pca(EEG);

    figure(40);
    subplot(2,2,1);
    bar(lambda);
    subplot(2,2,2);
    imagesc(V,[-1 1]); colorbar;

    subplot(2,2,3);
    bar(latent);
    subplot(2,2,4);
    imagesc(coeff,[-1 1]); colorbar;
    
% display covariance and correlation matrices of principal components

    figure(50);
    subplot(1,2,1);
    imagesc(cov(PCAcov),[-cov_scal cov_scal]); colorbar;
    title('covariance matrix of PCs');
    subplot(1,2,2);
    imagesc(corr(PCAcov),[-1 1]); colorbar;
    title('correlation matrix of PCs');

% reconstruct signals from all PCs
    
    EEG_rec = PCAcov*V';

    figure(60);
    subplot(1,3,1);
    hold on
    for i = 1:M
        plot(tme,EEG(:,i)/10/mean(std(EEG))-i);
    end
    ylim([-M-1 0]);
    title('original');
    subplot(1,3,2);
    hold on
    for i = 1:M
        plot(tme,EEG_rec(:,i)/10/mean(std(EEG_rec))-i);
    end
    ylim([-M-1 0]);
    title('reconstructed from all PCs');
    subplot(1,3,3);
    hold on
    for i = 1:M
        plot(tme,(EEG_rec(:,i)-EEG(:,i))/10/mean(std([EEG EEG_rec]))-i);
    end
    ylim([-M-1 0]);
    title('difference');

% reconstruct signals with some deleted PCs
    
    V_del            = V;
    V_del(:,PCA_del) = 0;
    EEG_rec_del      = PCAcov*V_del';

    figure(70);
    subplot(1,3,1);
    hold on
    for i = 1:M
        plot(tme,EEG(:,i)/10/mean(std(EEG))-i);
    end
    ylim([-M-1 0]);
    title('original');
    subplot(1,3,2);
    hold on
    for i = 1:M
        plot(tme,EEG_rec_del(:,i)/10/mean(std(EEG_rec_del))-i);
    end
    ylim([-M-1 0]);
    title('reconstructed without deleted PCs');
    subplot(1,3,3);
    hold on
    for i = 1:M
        plot(tme,(EEG_rec_del(:,i)-EEG(:,i))/10/mean(std([EEG EEG_rec_del]))-i);
    end
    ylim([-M-1 0]);
    title('difference');
    
end

