% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                       %
% %    Matlab function for lecture                                        %
% %    "Correlation analysis of multivariate time series and              %
% %     Principle Component Analysis"                                     %
% %    by Christian Rummel                                                %
% %                                                                       %
% %    BENESCO lecture series                                             %
% %    "Signal Analysis and Brain Oscillations in Health and Disease"     %
% %    Bern, 2016/04/01                                                   %
% %    Bern, 2017/03/31                                                   %
% %                                                                       %
% %    - windowed analysis of EEG data                                    %
% %    - properties of eigenvalues and eigenvectors are analyzed          %
% %    - spectral decomposition is introduced                             %
% %                                                                       %
% %    usage:                                                             %
% %    eig_val_vec (tme,EEG, T,dT, dspl)                                  %
% %                                                                       %
% %    input values:                                                      %
% %    tme    vector containing time from file start in seconds           %
% %    EEG    data matrix (L rows: temporal samples, M columns: chann.)   %
% %    T      size of analysis window in samples                          %
% %    dT     shift of analysis window in samples                         %
% %    dspl   flag for display of intermediate results (if >0)            %
% %                                                                       %
% %    literature:                                                        %
% %    - Müller et al. (2005), Phys. Rev. E71, 046116                     %
% %    - Müller et al. (2008), Europhys. Lett. 84, 10009                  %
% %    - Rummel et al. (2013), Neuroinformatics 11, 159-173               %
% %                                                                       %
% %    written by C. Rummel, 2016/03/23                                   %
% %    modified 2017/03/29                                                %
% %                                                                       %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function eig_val_vec (tme,EEG, T,dT, dspl)

    close all
    clc

    chan  = [1 13 25 33 41 47 53 59 64]; % channels selected for matrix display

    [L M] = size(EEG)           % L: number of rows, temporal samples
                                % M: number of columns, signal channels

% loop for windowed analysis of EEG data

    tme_evol    = [];
    lambda_evol = [];
    TCS_evol    = [];
    coll_evol   = [];
    symm_evol   = [];

    i_max = ceil(sqrt(M));
    j_max = ceil(M/i_max);
    
    t = 1;
    t_max = floor((L-T)/dT+1);

    for t = 1:t_max

% select data window
    
        tme_win  = tme((t-1)*dT+1:(t-1)*dT+T);
        tme_evol = [tme_evol tme_win(T/2)];
        EEG_win  = EEG((t-1)*dT+1:(t-1)*dT+T,:);
    
% optional display of normalized signals
    
        if dspl>0
            figure(10)
            hold on
            for i = 1:M
                if ismember(i,chan)
                    plot(tme_win,zscore(EEG_win(:,i))/5-i,'r');
                else
                    plot(tme_win,zscore(EEG_win(:,i))/5-i,'b');
                end
            end
        end

% calculate and diagonalize correlation matrix    

        corr_mat = corr(EEG_win);
        [V,D]    = eig(corr_mat);
        lambda   = D(1:M+1:M^2);
                
% evaluate "total correlation strength" (TCS),
% i.e. deviation of eigenvalues from 1 (equal distribution), 
% mind: sum(lambda) = M
% literature: Müller et al. (2008)
    
        TCS         = sum(abs(lambda-1))/2/(M-1);
        TCS_evol    = [TCS_evol TCS];
        lambda_evol = [lambda_evol lambda'];

        if dspl>0
            figure(20)
            bar(lambda-1);
            title('eigenvalue minus 1');
            text(0.75*M,5,strcat('TCS = ',num2str(TCS)));
        end
    
% evaluate "eigenvector collectivity" = "number of principal components"
% literature: Müller et al. (2005), Rummel et al. (2013)
    
        coll      = sum(V.^4);
        coll      = 1/M./coll;
        coll_evol = [coll_evol coll'];

        if dspl>0
            figure(30)
            subplot(1,2,1);
            imagesc(V,[-1 1]); colorbar;
            subplot(1,2,2);
            bar(coll);
            title('eigenvector collectivity');
        end
    
% evaluate "eigenvector symmetry"
% literature: Müller et al. (2008)

        symm      = abs(sum(V.*abs(V)));
        symm_evol = [symm_evol symm'];

        if dspl>0
            figure(40)
            subplot(1,2,1);
            imagesc(V,[-1 1]); colorbar;
            subplot(1,2,2);
            bar(symm);
            title('eigenvector symmetry');
        end
   
% optional:
% spectral decomposition        
        
%         spec_mat = zeros(M,M,M);
%         for k = 1:M
%             [i,j] = ind2sub([i_max,j_max],k);
%             spec_mat(:,:,k) = lambda(k)*V(:,k)*V(:,k)';
% 
%             figure(50)
%             axes('position',[0.05+(i-1)*0.95/i_max 1-j*0.95/j_max 0.9/i_max 0.9/j_max]);
%             imagesc(squeeze(spec_mat(:,:,k)),[-1 1]); 
%             set(gca,'XTick',[],'YTick',[]);
% 
%             figure(60)
%             axes('position',[0.05+(i-1)*0.95/i_max 1-j*0.95/j_max 0.9/i_max 0.9/j_max]);
%             imagesc(squeeze(spec_mat(:,:,k))); 
%             set(gca,'XTick',[],'YTick',[]);
%         end
% 
%         if dspl>0
%             figure(70)
%             imagesc(sum(spec_mat,3),[-1 1]); colorbar;
%         end
        
% wait for key press and continue with next window

        if dspl>0
            disp('press key to continue!');
            pause

            close all
        end
    end

% display temporal evolution of eigenvalues, TCS, collectivity and symmetry

    figure(80)

    subplot(5,1,1);
    plot(tme_evol,lambda_evol);
    set(gca,'XTick',floor(min(tme)):T*(tme(2)-tme(1)):ceil(max(tme)));
    ylabel('lambda');

    subplot(5,1,2);
    plot(tme_evol,log10(lambda_evol));
    set(gca,'XTick',floor(min(tme)):T*(tme(2)-tme(1)):ceil(max(tme)));
    ylabel('log10(lambda)');

    subplot(5,1,3);
    plot(tme_evol,TCS_evol);
    set(gca,'XTick',floor(min(tme)):T*(tme(2)-tme(1)):ceil(max(tme)));
%     ylim([0 1]);
    ylabel('TCS');

    subplot(5,1,4);
    hold on
    plot(tme_evol,coll_evol(1,:),'b');
    plot(tme_evol,coll_evol(2,:),'r');
    plot(tme_evol,coll_evol(3,:),'g');
    set(gca,'XTick',floor(min(tme)):T*(tme(2)-tme(1)):ceil(max(tme)));
%     ylim([0 1]);
    ylabel('collectivity');

    subplot(5,1,5);
    hold on
    plot(tme_evol,symm_evol(1,:),'b');
    plot(tme_evol,symm_evol(2,:),'r');
    plot(tme_evol,symm_evol(3,:),'g');
    set(gca,'XTick',floor(min(tme)):T*(tme(2)-tme(1)):ceil(max(tme)));
%     ylim([0 1]);
    ylabel('symmetry');    
    
end


